local _G = _G
local AVR = AVR
local math_rad = math.rad
local math_cos = math.cos
local math_sin = math.sin
local math_deg = math.deg
local GetPlayerFacing = GetPlayerFacing
local unpack = unpack

-- Isolate the environment
local folder, core = ...
setfenv(1, core)
-- local

local prev_OnEnable = core.OnEnable
function core:OnEnable()
	prev_OnEnable(self)
	AVR = _G.AVR
	if AVR then
		AddAVROptions()
	end
end

scene = {}
avrRangeRing = {}
avrTotemRing = {}
avrFireNovaRangeRing = {}
avrFireNovaColourRing = {}
function AddTotemToAVR(totemGUID, x, y, totemSlot, spellID, range, precise)--	/run TR.AddTotemToAVR()
	if AVR then
		if scene[totemGUID] then
			RemoveTotemFromAVR(totemGUID)
		end
		
		if AVR.AddCircleWarningEveryone then		
			scene[totemGUID] = AVR:GetTempScene("TR_"..totemGUID)
			scene[totemGUID]:ClearScene()
			scene[totemGUID].visible = true
			
			local bX, bY = GetAVRYardCoords(x, y)
	--~ 		Debug("avr07","bX, bY", bX, bY)
			local range = range or totemInfo[spellID].range or 3
			
			local m = AVRCircleMesh:New(range, 30, false, P.AVRLineThickness * 32)
			m:TranslateMesh(bX, bY)
			m:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, 1)
			scene[totemGUID]:AddMesh(m)
			avrRangeRing[totemGUID] = m
			
			if P.AVRTotemColourRing == true then
				local m = AVRCircleMesh:New(range-0.2, 30, true, P.AVRLineThickness * 32)
				m:TranslateMesh(bX, bY)
				m:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, 1) --P.totemColour[totemSlot].a
				scene[totemGUID]:AddMesh(m)
				avrTotemRing[totemGUID] = m
			end
			
			
			--[[ Filled in circle coloured to the totem colour. I'm commenting it out to save CPU cycles.]]
			if P.AVRSolidCircle == true then
				local m = AVRFilledCircleMesh:New(range or 3,30, false)
				m:TranslateMesh(bX, bY)
				m:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, (-0.0033333333*range+0.18333333))
				--(-0.005*range+0.2)) --10y = 15% alpha, 30y = .05% alpha.
				
				scene[totemGUID]:AddMesh(m)
			end
			
			--[[ Small texture under totem.]]
			if P.AVRTotemLocation == true then
				local m = AVRFilledCircleMesh:New(1,6, false, nil)
				m:TranslateMesh(bX, bY)
				m:SetColor(P.totemColour[totemSlot].r, P.totemColour[totemSlot].g, P.totemColour[totemSlot].b, .2)
				scene[totemGUID]:AddMesh(m)
			end

			if totemSlot == 1 and P.avrFireNovaRing == true then
				if precise == true then
					range = 10
				else
					range = range - spawnDistance
				end
				
				local m = AVRCircleMesh:New(range, 30, false, P.AVRLineThickness * 32)
				m:TranslateMesh(bX, bY)
				m:SetColor(P.avrFireNovaColour.r, P.avrFireNovaColour.g, P.avrFireNovaColour.b, 1)
				scene[totemGUID]:AddMesh(m)
				avrFireNovaRangeRing[totemGUID] = avrFireNovaRangeRing[totemGUID] or {}
				avrFireNovaRangeRing[totemGUID].mesh = m
				avrFireNovaRangeRing[totemGUID].range = range
				
				
				if P.AVRTotemColourRing == true then
					local m = AVRCircleMesh:New(range-0.2, 30, true, P.AVRLineThickness * 32)
					m:TranslateMesh(bX, bY)
					m:SetColor(P.avrFireNovaColour.r, P.avrFireNovaColour.g, P.avrFireNovaColour.b, 1) --P.totemColour[totemSlot].a
					scene[totemGUID]:AddMesh(m)
					
					avrFireNovaColourRing[totemGUID] = m
--~ 					avrFireNovaColourRing[totemGUID].mesh = m
--~ 					avrFireNovaColourRing[totemGUID].range = range
					
					
				end
				
				
			end
			
			
--~ 			for a, b in pairs(scene[totemGUID].meshes) do 
--~ 				Debug("AddTotemToAVR", a, b)
--~ 				b:SetColor(1,1,1,1)
--~ 			end
			
			
		end
	end
end

--------------------------------------------------
function GetAVRYardCoords(aX, aY)				--
-- AVR converts numbers differently then I do.	--
-- Thier X is the same, but Y is not. 			--
--------------------------------------------------
	local sX, sY = unpack(AVR.zoneData.currentScale)
	return aX*sX,(1-aY)*sY
end

function RemoveTotemFromAVR(totemGUID)--	/run TR.RemoveTotemFromAVR()
	if AVR and scene[totemGUID] then
		scene[totemGUID]:ClearScene()
		scene[totemGUID].visible = false
		scene[totemGUID] = nil
	end
end
